package com.hgny.protobuf.utils;

import com.google.gson.Gson;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
import org.apache.commons.compress.compressors.bzip2.BZip2CompressorInputStream;
import org.apache.commons.compress.compressors.bzip2.BZip2CompressorOutputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
import org.apache.commons.compress.utils.IOUtils;

import java.io.*;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

public class Tools {

  public static int parseTimestamp(long timestamp) {
    String t = String.valueOf(timestamp);
    return Integer.parseInt(t.substring(0, t.length() - 3));
  }

  public static String extraTarGz(File file, String path) {
    if (!file.getName().contains(".tar.gz")) {
      return null;
    }
    try (TarArchiveInputStream inputStream = new TarArchiveInputStream(
        new GZIPInputStream(
            new FileInputStream(file)
        )
    )) {
      return extraTar(inputStream, path);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public static String extraTarBz2(File file, String path) {
    if (!file.getName().contains(".tar.bz2")) {
      return null;
    }
    try (TarArchiveInputStream inputStream = new TarArchiveInputStream(
        new BZip2CompressorInputStream(
            new FileInputStream(file)
        )
    )) {
      return extraTar(inputStream, path);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  private static String extraTar(TarArchiveInputStream inputStream, String path) throws IOException {
    ArchiveEntry entry = inputStream.getNextEntry();
    String outName = entry.getName();
    while (entry != null) {
      String entryName = entry.getName();
      File tmpFile = new File(path + "/" + entryName);
      if (tmpFile.exists()) {
        entry = inputStream.getNextEntry();
        continue;
      }
      if (entry.isDirectory()) {
        createDir(path, entryName);
      } else {
        createDir(tmpFile.getParent() + "/", null);
        try (FileOutputStream outputStream = new FileOutputStream(tmpFile)) {
          int read = 0;
          byte[] buffer = new byte[1024];
          while ((read = inputStream.read(buffer, 0 ,buffer.length)) != -1) {
            outputStream.write(buffer, 0, read);
          }
        } catch (IOException e) {
          throw new RuntimeException(e);
        }
      }
      entry = inputStream.getNextEntry();
    }
    return outName;
  }

  @SuppressWarnings("ALL")
  public static List<String> dataParser(File file) throws IOException {
    List<String> result = new ArrayList<>();
    FileReader fileReader = new FileReader(file);
    BufferedReader bufferedReader = new BufferedReader(fileReader);
    String line = null;
    Gson gson = new Gson();
    DateTimeFormatter formatter = new DateTimeFormatterBuilder()
        .appendPattern("yyyy-MM-dd HH:mm:ss")
        .toFormatter();
    while ((line = bufferedReader.readLine()) != null) {
      Map<String, Object> map = gson.fromJson(line, Map.class);
      Map<String, Object> data = (Map<String, Object>) map.get("data");
      String time = (String) map.get("time");
      LocalDateTime parse = LocalDateTime.parse(time, formatter);
      long timestamp = parse.toInstant(ZoneOffset.ofHours(8)).toEpochMilli();
      int parseTimestamp = Tools.parseTimestamp(timestamp);
      data.put("time", parseTimestamp);
      String s = Constant.KEYS.stream().map((key) -> {
        Object o = data.get(key);
        if (o == null) {
          return "";
        }
        return String.valueOf(o);
      }).collect(Collectors.joining(","));
      result.add(s);
    }
    return result;
  }

  public static void compressBZip2(String data, File file) {
    try (FileOutputStream outputStream = new FileOutputStream(file);
         BZip2CompressorOutputStream zip = new BZip2CompressorOutputStream(outputStream, 9)
    ) {
      zip.write(data.getBytes(StandardCharsets.UTF_8));
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public static void compressTarGz(String path, File file) throws IOException {
    createTarGZ(path, file);
  }

  public static void createTarGZ(String path, File file)
      throws IOException
  {
    try (
        FileOutputStream fOut = new FileOutputStream(file);
        BufferedOutputStream bOut = new BufferedOutputStream(fOut);
        GzipCompressorOutputStream gzOut = new GzipCompressorOutputStream(bOut);
        TarArchiveOutputStream tOut = new TarArchiveOutputStream(gzOut)
        ) {
      addFileToTarGz(tOut, path, "");
      tOut.flush();
    }
  }

  private static void addFileToTarGz(TarArchiveOutputStream tOut, String path, String base)
      throws IOException
  {
    File f = new File(path);
    String entryName = base + f.getName();
    TarArchiveEntry tarEntry = new TarArchiveEntry(f, entryName);
    tOut.putArchiveEntry(tarEntry);

    if (f.isFile()) {
      IOUtils.copy(new FileInputStream(f), tOut);
      tOut.flush();
      tOut.closeArchiveEntry();
    } else {
      tOut.closeArchiveEntry();
      File[] children = f.listFiles();
      if (children != null) {
        for (File child : children) {
          addFileToTarGz(tOut, child.getAbsolutePath(), entryName + "/");
        }
      }
    }
  }

  public static void createDir(String path, String subPath) {
    File file = new File(path);
    if (!(subPath == null || subPath.trim().equals(""))) {
      file = new File(path + "/" + subPath);
    }
    if (!file.exists()) {
      file.mkdirs();
    }
  }

}
